package speiger.src.coreengine.rendering.textures.custom;

import org.lwjgl.stb.STBImage;
import org.lwjgl.system.MemoryUtil;

import speiger.src.coreengine.math.misc.ColorSpaces;

public class Drawable implements IDrawable, AutoCloseable {
	int width;
	int height;
	long pixels;
	boolean isSTB;
	
	public Drawable(int width, int height) {
		this(width, height, MemoryUtil.nmemAllocChecked(4L * width * height), false);
	}
	
	public Drawable(int width, int height, long pixels, boolean isSTB) {
		this.width = width;
		this.height = height;
		this.pixels = pixels;
		this.isSTB = isSTB;
	}
	
	@Override
	public ColorSpaces colorspace() { return ColorSpaces.ABGR; }
	@Override
	public int width() { return width; }
	@Override
	public int height() { return height; }
	public long pixels() { return pixels; }
	public boolean isSTBImage() { return isSTB; }
	
	@Override
	public void close() {
		if(pixels == 0L) return;
		if(isSTB) STBImage.nstbi_image_free(pixels);
		else MemoryUtil.nmemFree(pixels);
		pixels = 0L;
	}
	
	protected long offset(int x, int y) {
		return ((y * width()) + x) << 2;
	}
	
	protected void ensureValid(int index) {
		ensureValid(index % width, index / width);
	}
	
	protected void ensureValid(int x, int y) {
		if(x < 0 || y < 0) throw new ArrayIndexOutOfBoundsException("Index out of bounds: X=["+x+"], Y=["+y+"]");
		if(x >= width || y >= height) throw new ArrayIndexOutOfBoundsException("Index out of bounds: X=["+x+"], Y=["+y+"], width=["+width+"], height=["+height+"]");
		if(pixels == 0L) throw new IllegalStateException("Pixel Data doesn't exist");
	}
	
	@Override
	public void set(int index, int data) {
		ensureValid(index);
		MemoryUtil.memPutInt(index * 4L, data);
	}

	@Override
	public void setR(int index, int red) {
		ensureValid(index);
		MemoryUtil.memPutByte(index * 4L, (byte)(red & 0xFF));
	}

	@Override
	public void setG(int index, int green) {
		ensureValid(index);
		MemoryUtil.memPutByte(index * 4L + 1L, (byte)(green & 0xFF));
	}

	@Override
	public void setB(int index, int blue) {
		ensureValid(index);
		MemoryUtil.memPutByte(index * 4L + 2L, (byte)(blue & 0xFF));
	}

	@Override
	public void setA(int index, int alpha) {
		ensureValid(index);
		MemoryUtil.memPutByte(index * 4L + 3L, (byte)(alpha & 0xFF));
	}

	@Override
	public void fill(int x, int y, int width, int height, int data) {
		ensureValid(x, y);
		ensureValid(x+width, y+height);
		for(int yOff = 0;yOff<height;yOff++) {
			MemoryUtil.memSet(offset(x, y+yOff), data, width * 4L);
		}
	}
	
	@Override
	public int get(int index) {
		ensureValid(index);
		return MemoryUtil.memGetInt(index * 4L);
	}
	
	@Override
	public int getR(int index) {
		ensureValid(index);
		return MemoryUtil.memGetByte(index * 4L);
	}
	
	@Override
	public int getG(int index) {
		ensureValid(index);
		return MemoryUtil.memGetByte(index * 4L + 1L);
	}
	
	@Override
	public int getB(int index) {
		ensureValid(index);
		return MemoryUtil.memGetByte(index * 4L + 2L);
	}
	
	@Override
	public int getA(int index) {
		ensureValid(index);
		return MemoryUtil.memGetByte(index * 4L + 3L);
	}
}
